@@ -12,7 +12,7 @@ from flask import Flask, Response
 from flask.views import MethodViewType
 from werkzeug.serving import is_running_from_reloader
 from prometheus_client import Counter, Histogram, Gauge, Summary
-from prometheus_client import generate_latest, CONTENT_TYPE_LATEST
+from prometheus_client.exposition import choose_encoder
 
 if sys.version_info[0:2] >= (3, 4):
     # Python v3.4+ has a built-in has __wrapped__ attribute
@@ -105,6 +105,7 @@ class PrometheusMetrics(object):
     def __init__(self, app, path='/metrics',
                  export_defaults=True, defaults_prefix='flask',
                  group_by='path', buckets=None,
+                 default_latency_as_histogram=True,
                  default_labels=None, response_converter=None,
                  excluded_paths=None, registry=None, **kwargs):
         """
@@ -123,6 +124,8 @@ class PrometheusMetrics(object):
             (defaults to `path`)
         :param buckets: the time buckets for request latencies
             (will use the default when `None`)
+        :param default_latency_as_histogram: export request latencies
+            as a Histogram (defaults), otherwise use a Summary
         :param default_labels: default labels to attach to each of the
             metrics exposed by this `PrometheusMetrics` instance
         :param response_converter: a function that converts the captured
@@ -137,6 +140,7 @@ class PrometheusMetrics(object):
         self._export_defaults = export_defaults
         self._defaults_prefix = defaults_prefix or 'flask'
         self._default_labels = default_labels or {}
+        self._default_latency_as_histogram = default_latency_as_histogram
         self._response_converter = response_converter or make_response
         self.buckets = buckets
         self.version = __version__
@@ -224,8 +228,9 @@ class PrometheusMetrics(object):
 
         if self._export_defaults:
             self.export_defaults(
-                self.buckets, self.group_by,
-                self._defaults_prefix, app
+                buckets=self.buckets, group_by=self.group_by,
+                latency_as_histogram=self._default_latency_as_histogram,
+                prefix=self._defaults_prefix, app=app
             )
 
     def register_endpoint(self, path, app=None):
@@ -260,7 +265,8 @@ class PrometheusMetrics(object):
             if 'prometheus_multiproc_dir' in os.environ:
                 multiprocess.MultiProcessCollector(registry)
 
-            headers = {'Content-Type': CONTENT_TYPE_LATEST}
+            generate_latest, content_type = choose_encoder(request.headers.get("Accept"))
+            headers = {'Content-Type': content_type}
             return generate_latest(registry), 200, headers
 
     def start_http_server(self, port, host='0.0.0.0', endpoint='/metrics'):
@@ -289,10 +295,12 @@ class PrometheusMetrics(object):
         thread.start()
 
     def export_defaults(self, buckets=None, group_by='path',
+                        latency_as_histogram=True,
                         prefix='flask', app=None, **kwargs):
         """
         Export the default metrics:
             - HTTP request latencies
+            - HTTP request exceptions
             - Number of HTTP requests
 
         :param buckets: the time buckets for request latencies
@@ -300,6 +308,9 @@ class PrometheusMetrics(object):
         :param group_by: group default HTTP metrics by
             this request property, like `path`, `endpoint`, `rule`, etc.
             (defaults to `path`)
+        :param latency_as_histogram: export request latencies
+            as a Histogram, otherwise use a Summary instead
+            (defaults to `True` to export as a Histogram)
         :param prefix: prefix to start the default metrics names with
             or `NO_PREFIX` (to skip prefix)
         :param app: the Flask application
@@ -311,11 +322,6 @@ class PrometheusMetrics(object):
         if not prefix:
             prefix = self._defaults_prefix or 'flask'
 
-        # use the default buckets from prometheus_client if not given here
-        buckets_as_kwargs = {}
-        if buckets is not None:
-            buckets_as_kwargs['buckets'] = buckets
-
         if kwargs.get('group_by_endpoint') is True:
             warnings.warn(
                 'The `group_by_endpoint` argument of '
@@ -342,27 +348,53 @@ class PrometheusMetrics(object):
         else:
             prefix = prefix + "_"
 
+        try:
+            self.info(
+                '%sexporter_info' % prefix,
+                'Information about the Prometheus Flask exporter',
+                version=self.version
+            )
+        except ValueError:
+            return  # looks like we have already exported the default metrics
+
         labels = self._get_combined_labels(None)
 
-        histogram = Histogram(
-            '%shttp_request_duration_seconds' % prefix,
-            'Flask HTTP request duration in seconds',
-            ('method', duration_group_name, 'status') + labels.keys(),
-            registry=self.registry,
-            **buckets_as_kwargs
-        )
+        if latency_as_histogram:
+            # use the default buckets from prometheus_client if not given here
+            buckets_as_kwargs = {}
+            if buckets is not None:
+                buckets_as_kwargs['buckets'] = buckets
 
-        counter = Counter(
+            request_duration_metric = Histogram(
+                '%shttp_request_duration_seconds' % prefix,
+                'Flask HTTP request duration in seconds',
+                ('method', duration_group_name, 'status') + labels.keys(),
+                registry=self.registry,
+                **buckets_as_kwargs
+            )
+
+        else:
+            # export as Summary instead
+            request_duration_metric = Summary(
+                '%shttp_request_duration_seconds' % prefix,
+                'Flask HTTP request duration in seconds',
+                ('method', duration_group_name, 'status') + labels.keys(),
+                registry=self.registry
+            )
+
+        counter_labels = ('method', 'status') + labels.keys()
+        request_total_metric = Counter(
             '%shttp_request_total' % prefix,
             'Total number of HTTP requests',
-            ('method', 'status') + labels.keys(),
+            counter_labels,
             registry=self.registry
         )
 
-        self.info(
-            '%sexporter_info' % prefix,
-            'Information about the Prometheus Flask exporter',
-            version=self.version
+        request_exceptions_metric = Counter(
+            '%shttp_request_exceptions_total' % prefix,
+            'Total number of HTTP requests which resulted in an exception',
+            counter_labels,
+            registry=self.registry
         )
 
         def before_request():
@@ -384,16 +416,16 @@ class PrometheusMetrics(object):
                 else:
                     group = getattr(request, duration_group)
 
-                histogram_labels = {
+                request_duration_labels = {
                     'method': request.method,
                     'status': _to_status_code(response.status_code),
                     duration_group_name: group
                 }
-                histogram_labels.update(labels.values_for(response))
+                request_duration_labels.update(labels.values_for(response))
 
-                histogram.labels(**histogram_labels).observe(total_time)
+                request_duration_metric.labels(**request_duration_labels).observe(total_time)
 
-            counter.labels(
+            request_total_metric.labels(
                 method=request.method, status=_to_status_code(response.status_code),
                 **labels.values_for(response)
             ).inc()
@@ -410,24 +442,29 @@ class PrometheusMetrics(object):
 
             response = make_response('Exception: %s' % exception, 500)
 
+            if callable(duration_group):
+                group = duration_group(request)
+            else:
+                group = getattr(request, duration_group)
+
+            request_exceptions_metric.labels(
+                method=request.method, status=500,
+                **labels.values_for(response)
+            ).inc()
+
             if hasattr(request, 'prom_start_time'):
                 total_time = max(default_timer() - request.prom_start_time, 0)
 
-                if callable(duration_group):
-                    group = duration_group(request)
-                else:
-                    group = getattr(request, duration_group)
-
-                histogram_labels = {
+                request_duration_labels = {
                     'method': request.method,
                     'status': 500,
                     duration_group_name: group
                 }
-                histogram_labels.update(labels.values_for(response))
+                request_duration_labels.update(labels.values_for(response))
 
-                histogram.labels(**histogram_labels).observe(total_time)
+                request_duration_metric.labels(**request_duration_labels).observe(total_time)
 
-            counter.labels(
+            request_total_metric.labels(
                 method=request.method, status=500,
                 **labels.values_for(response)
             ).inc()
@@ -793,11 +830,43 @@ class ConnexionPrometheusMetrics(PrometheusMetrics):
     Specific extension for Connexion (https://connexion.readthedocs.io/)
     that makes sure responses are converted to Flask responses.
     """
-    def __init__(self, app, **kwargs):
-        from connexion.apis.flask_api import FlaskApi
+    def __init__(self, app, default_mimetype='application/json', **kwargs):
+        flask_app = app.app if app else None
         if 'response_converter' not in kwargs:
-            kwargs['response_converter'] = FlaskApi.get_response
-        super().__init__(app, **kwargs)
+            kwargs['response_converter'] = self._create_response_converter(default_mimetype)
+
+        super(ConnexionPrometheusMetrics, self).__init__(flask_app, **kwargs)
+
+    @staticmethod
+    def content_type(content_type):
+        """
+        Force the content type of the response,
+        which would be otherwise overwritten by the metrics conversion
+        to application/json.
+
+        :param content_type: the value to send in the
+          Content-Type response header
+        """
+
+        def decorator(f):
+            @wraps(f)
+            def func(*args, **kwargs):
+                request.prom_connexion_content_type = content_type
+                return f(*args, **kwargs)
+            return func
+        return decorator
+
+    @staticmethod
+    def _create_response_converter(default_mimetype):
+        def _make_response(response):
+            from connexion.apis.flask_api import FlaskApi
+
+            mimetype = default_mimetype
+            if hasattr(request, 'prom_connexion_content_type'):
+                mimetype = request.prom_connexion_content_type
+
+            return FlaskApi.get_response(response, mimetype=mimetype)
+        return _make_response
 
 
 class RESTfulPrometheusMetrics(PrometheusMetrics):
@@ -814,9 +883,18 @@ class RESTfulPrometheusMetrics(PrometheusMetrics):
         :param api: the Flask-RESTful API instance
         """
 
-        if 'response_converter' not in kwargs:
+        if api and 'response_converter' not in kwargs:
             kwargs['response_converter'] = self._create_response_converter(api)
-        super().__init__(app, **kwargs)
+        super(RESTfulPrometheusMetrics, self).__init__(app, **kwargs)
+
+    @classmethod
+    def for_app_factory(cls, api=None, **kwargs):
+        return cls(app=None, api=api, **kwargs)
+
+    def init_app(self, app, api=None):
+        if api:
+            self._response_converter = self._create_response_converter(api)
+        return super(RESTfulPrometheusMetrics, self).init_app(app)
 
     @staticmethod
     def _create_response_converter(api):
@@ -827,4 +905,4 @@ class RESTfulPrometheusMetrics(PrometheusMetrics):
         return _make_response
 
 
-__version__ = '0.15.0'
+__version__ = '0.17.0'
